@@ -10,15 +10,14 @@ use POE qw( Wheel::Run );
 use Term::ANSIColor qw(:constants);
 use Text::Balanced;
 
-our $VERSION = '2.0207_1';
-our @DEFAULT_SHELL_COMMAND = (ssh => '-o', 'BatchMode yes', '-o', 'StrictHostKeyChecking no', '-o', 'ConnectTimeout 20', '[%u]-l', '[]%u', '%h');
+our $VERSION = '2.0100';
+our @DEFAULT_SHELL_COMMAND = (ssh => '-o', 'BatchMode yes', '-o', 'StrictHostKeyChecking no', '-o', 'ConnectTimeout 20', '%h');
 
 # new {{{
 sub new {
-    my $class = shift;
-    my $this  = bless { hosts=>[], cmd=>[], _shell_cmd=>[@DEFAULT_SHELL_COMMAND] }, $class;
+    my $this = bless { hosts=>[], cmd=>[], _shell_cmd=>[@DEFAULT_SHELL_COMMAND] };
 
-    return $this;
+    $this;
 }
 # }}}
 
@@ -38,29 +37,22 @@ sub _process_space_delimited {
             $that = $rem;
 
         } else {
-            my ($tok, $rem) = split ' ', $that, 2; 
+            my ($tok, $rem) = split m/\s+/, $that, 2; 
 
             push @output, $tok;
             $that = $rem;
         }
     }
 
-    return @output
+    @output
 }
 # }}}
 # _process_hosts {{{
 sub _process_hosts {
     my $this = shift;
-
     my @h = do {
         my @tmp = map { my $k = $_; $k =~ s/^\@// ? @{$this->{groups}{$k} or die "couldn't find group: \@$k\n"} : $_ } @_;
         my %h; @h{@tmp} = ();
-        for(keys %h) {
-            if(my ($k) = m/^\-(.+)/) {
-                delete $h{$_};
-                delete $h{$k};
-            }
-        }
         sort keys %h;
     };
 
@@ -68,17 +60,16 @@ sub _process_hosts {
     for( map { length $this->_host_route_to_nick($_) } @h ) {
         $l = $_ if $_>$l
     }
-
     $this->{_host_width} = $l if $l != $o;
 
-    return @h;
+    @h;
 }
 # }}}
 # _host_route_to_nick {{{
 sub _host_route_to_nick {
     my $this = shift;
 
-    return join "", shift =~ m/(?:!|[^!]+$)/g
+    join "", shift =~ m/(?:!|[^!]+$)/g
 }
 # }}}
 
@@ -94,57 +85,24 @@ sub set_shell_command_option {
         $this->{_shell_cmd} = [ $this->_process_space_delimited($arg||"") ];
     }
 
-    return $this;
+    $this;
 }
 # }}}
 # set_group_option {{{
 sub set_group_option {
-    my $this   = shift;
-    my $groups = ($this->{groups} ||= {});
+    my $this  = shift;
 
     my ($name, $value);
     while( ($name, $value) = splice @_, 0, 2 and $name and $value ) {
         if( ref($value) eq "ARRAY" ) {
-            $groups->{$name} = [ @$value ]; # make a real copy
+            $this->{groups}{$name} = [ @$value ]; # make a real copy
 
         } else {
-            $groups->{$name} = [ $this->_process_space_delimited( $value ) ];
+            $this->{groups}{$name} = [ $this->_process_space_delimited( $value ) ];
         }
     }
 
-    my @groups = keys %{ $this->{groups} };
-    my $replace_limit = 30;
-    REPLACE_GROPUS: {
-        my $replaced = 0;
-
-        for my $group (@groups) {
-            my $hosts = $groups->{$group};
-
-            my $r = 0;
-            for(@$hosts) {
-                if( m/^@(.+)/ ) {
-                    if( my $g = $groups->{$1} ) {
-                        $_ = $g;
-
-                        $r ++;
-                    }
-                }
-            }
-
-            if( $r ) {
-                my %h;
-                @h{ map {ref $_ ? @$_ : $_} @$hosts } = ();
-                $groups->{$group} = [ keys %h ];
-                $replaced ++;
-            }
-        }
-
-        $replace_limit --;
-        last if $replace_limit < 1;
-        redo if $replaced;
-    }
-
-    return $this;
+    $this;
 }
 # }}}
 # set_logfile_option {{{
@@ -154,7 +112,7 @@ sub set_logfile_option {
     my $trunc = shift;
 
     unless( our $already_compiled++ ) {
-        my $load_ansi_filter_package = q {
+        eval q {
             package App::MrShell::ANSIFilter;
             use Symbol;
             use Tie::Handle;
@@ -176,17 +134,13 @@ sub set_logfile_option {
                 $pfft;
             }
 
-        1};
-
-        eval $load_ansi_filter_package or die $@; ## no critic -- sometimes this kind of eval is ok
-        # (This probably isn't one of them.)
+        1} or die $@;
     }
 
-    open my $log, ($trunc ? ">" : ">>"), $file or croak "couldn't open $file for write: $!"; ## no critic -- I mean to pass this around, shut up
+    open my $log, ($trunc ? ">" : ">>"), $file or croak "couldn't open $file for write: $!";
 
     $this->{_log_fh} = App::MrShell::ANSIFilter::filtered_handle($log);
-
-    return $this;
+    $this;
 }
 # }}}
 # set_debug_option {{{
@@ -204,17 +158,15 @@ sub set_debug_option {
     }
 
     $this->{debug} = $val ? $val : 1;
-
-    return $this;
+    $this;
 }
 # }}}
 # set_no_command_escapes_option {{{
 sub set_no_command_escapes_option {
     my $this = shift;
 
-    $this->{no_command_escapes} = shift || 0;
-
-    return $this;
+    $this->{no_command_escapes} = 1;
+    $this;
 }
 # }}}
 
@@ -228,19 +180,15 @@ sub groups {
 # }}}
 
 # set_usage_error($&) {{{
-sub set_usage_error($&) { ## no critic -- prototypes are bad how again?
+sub set_usage_error($&) {
     my $this = shift;
     my $func = shift;
     my $pack = caller;
     my $name = $pack . "::$func";
     my @args = @_;
 
-    $this->{_usage_error} = sub {
-        no strict 'refs'; ## no critic -- how would you call this by name without this?
-        $name->(@args)
-    };
-
-    return $this;
+    $this->{_usage_error} = sub { no strict 'refs'; $name->(@args) };
+    $this;
 }
 # }}}
 # read_config {{{
@@ -254,25 +202,25 @@ sub read_config {
     }
 
     if( my $c = $this->{_conf}{options}{'shell-command'} ) {
-        $this->set_shell_command_option( $c );
+        $this->set_shell_command_option( 1, $c );
     }
 
     if( my $c = $this->{_conf}{options}{'logfile'} ) {
         my $t = $this->{_conf}{options}{'truncate-logfile'};
-        my $v = ($t ? 1:0);
-           $v = 0 if $t =~ m/(?:no|false)/i;
+        my $v = 1 if $t;
+           $v = 0 if $c =~ m/(?:no|false)/i;
 
-        $this->set_logfile_option($c, $v);
+        $this->set_logfile_option($c, $t);
     }
 
     if( my $c = $this->{_conf}{options}{'no-command-escapes'} ) {
-        my $v = ($c ? 1:0);
+        my $v = 1 if $c;
            $v = 0 if $c =~ m/(?:no|false)/i;
 
         $this->set_no_command_escapes_option( $v );
     }
 
-    return $this;
+    $this;
 }
 # }}}
 # set_hosts {{{
@@ -280,8 +228,7 @@ sub set_hosts {
     my $this = shift;
 
     $this->{hosts} = [ $this->_process_hosts(@_) ];
-
-    return $this;
+    $this;
 }
 # }}}
 # queue_command {{{
@@ -308,7 +255,7 @@ sub queue_command {
         push @{$this->{_cmd_queue}{$h}}, [@_]; # make a real copy
     }
 
-    return $this;
+    $this;
 }
 # }}}
 # run_queue {{{
@@ -326,7 +273,7 @@ sub run_queue {
 
     POE::Kernel->run();
 
-    return $this;
+    $this
 }
 # }}}
 
@@ -342,7 +289,7 @@ sub std_msg {
     my $time_str = strftime('%H:%M:%S', localtime);
 
     print $time_str,
-        sprintf(' %-*s', $this->{_host_width}+2, $host_msg),
+        sprintf(' %4s %-*s', "[$cmdno]", $this->{_host_width}+2, $host_msg),
             ( $fh==2 ? ('[',BOLD,YELLOW,'stderr',RESET,'] ') : () ), $msg, RESET, "\n";
 
     if( $this->{_log_fh} ) {
@@ -351,10 +298,10 @@ sub std_msg {
         # No point in printing colors, stripped anyway.  Formatting columns is
         # equally silly -- in append mode anyway.
         $host_msg = $host ? "$host: " : "";
-        print {$this->{_log_fh}} "$time_str $host_msg", ($fh==2 ? "[stderr] " : ""), $msg, "\n";
+        print {$this->{_log_fh}} "$time_str [$cmdno] $host_msg", ($fh==2 ? "[stderr] " : ""), $msg, "\n";
     }
 
-    return $this;
+    $this;
 }
 # }}}
 
@@ -367,8 +314,6 @@ sub line {
 
     $$lineno ++;
     $this->std_msg($host, $cmdno, $fh, $line);
-
-    return;
 }
 # }}}
 
@@ -379,8 +324,6 @@ sub _sigchld_exit_error {
     $exit >>= 8;
 
     $this->std_msg("?", -1, 0, BOLD.RED."-- sigchld received for untracked pid($pid, $exit), probably a bug in Mr. Shell --");
-
-    return;
 }
 
 sub sigchld {
@@ -406,8 +349,6 @@ sub sigchld {
     }
 
     $_[KERNEL]->yield( stall_close => $kid->ID, 0 );
-
-    return;
 }
 # }}}
 # _close {{{
@@ -436,8 +377,6 @@ sub _close {
     } else {
         $_[KERNEL]->yield( stall_close => $wid, $count+1 );
     }
-
-    return;
 }
 # }}}
 # error_event {{{
@@ -447,10 +386,8 @@ sub error_event {
     my ($kid, $host, $cmdno, @c) = @{ delete $this->{_wid}{$wid} || return };
     delete $this->{_pid}{ $kid->PID };
 
-    $errstr = "remote end closed" if $operation eq "read" and not $errnum;
+    $errstr = "remote end closed" if $operation eq "read" and !$errnum;
     $this->std_msg($host, $cmdno, 0, RED."-- $operation error $errnum: $errstr --");
-
-    return;
 }
 # }}}
 
@@ -462,7 +399,7 @@ sub set_subst_vars {
         $this->{_subst}{$k} = $v unless exists $this->{_subst}{$k};
     }
 
-    return $this;
+    $this;
 }
 # }}}
 # subst_cmd_vars {{{
@@ -476,18 +413,15 @@ sub subst_cmd_vars {
                 #  proovably true that I do.)
 
     if( $host =~ m/\b(?!<\\)!/ ) {
+        delete $h{'%h'};
         my @hosts = split '!', $host;
 
-        my @indexes_of_replacements;
         for(my $i=0; $i<@c; $i++) {
             if( $c[$i] eq '%h' ) {
                 splice @c, $i, 1, $hosts[0];
 
-                push @indexes_of_replacements, $i;
-
                 for my $h (reverse @hosts[1 .. $#hosts]) {
                     splice @c, $i+1, 0, @c[0 .. $i-1] => $h;
-                    push @indexes_of_replacements, $i+1 + $indexes_of_replacements[-1];
 
                     unless( $this->{no_command_escapes} ) {
                         for my $arg (@c[$i+1 .. $#c]) {
@@ -506,45 +440,20 @@ sub subst_cmd_vars {
             }
         }
 
-        my $beg = 0;
-        for my $i (@indexes_of_replacements) {
-            if( $c[$i] =~ s/^([\w.\-_]+)@// ) {
-                my $u = $1;
-                for(@c[$beg .. $i-1]) {
-                    s/^(\[\%u\]|\[\](?=\%u))//;
-                    $_ = $u if $_ eq '%u';
-                }
-
-            } else {
-                # NOTE: there's really no need to go through and remove [%u]
-                # conditional options, they'll automatically get nuked below
-                $c[$i] =~ s/\\@/@/g;
-            }
-            $beg = $i+1;
-        }
-
-        delete $h{'%h'};
-
     } else {
         $h{'%h'} =~ s/\\!/!/g;
     }
 
-    if( $h{'%h'} ) {
-        $h{'%u'} = $1 if $h{'%h'} =~ s/^([\w.\-_]+)@//;
-        $h{'%h'} =~ s/\\@/@/g;
-    }
-
-    @c = map {exists $h{$_} ? $h{$_} : $_}
-         map { m/^\[([^\[\]]+)\]/ ? ($h{$1} ? do{s/^\[\Q$1\E\]//; $_} : ()) : ($_) } ## no critic: why on earth not?
-         map { s/\[\]\%(\w+)/[\%$1]\%$1/; $_ }                                       ## no critic: why on earth not?
-         @c;
-
     if( $this->{debug} ) {
-        local $" = ")(";
-        $this->std_msg($host, $h{'%n'}, 0, BOLD.BLACK."DEBUG: exec(@c)");
+        my @cmd = map {exists $h{$_} ? $h{$_} : $_} @c;
+
+        my @dt = map {"<$_>"} @cmd;
+        $this->std_msg($host, $h{'%n'}, 0, BOLD.BLACK."DEBUG: exec(@dt)");
+
+        return @cmd;
     }
 
-    return @c;
+    map {exists $h{$_} ? $h{$_} : $_} @c;
 }
 # }}}
 # start_queue_on_host {{{
@@ -569,8 +478,6 @@ sub start_queue_on_host {
     my $lineno = 0;
     my $info = [ $kid, $host, $cmdno, \$lineno, @next ];
     $this->{_wid}{ $kid->ID } = $this->{_pid}{ $kid->PID } = $info;
-
-    return;
 }
 # }}}
 
@@ -599,9 +506,6 @@ sub poe_start {
     }
 
     delete $this->{_cmd_queue};
-
     return;
 }
 # }}}
-
-1;
